@@ -10,7 +10,9 @@ PANIC = 60
 logging.addLevelName(PANIC, "PANIC")
 logging.addLevelName(logging.CRITICAL, "FATAL")
 
-
+# ---------------------------------------------------------------------------
+#   ShinnyLogger
+# ---------------------------------------------------------------------------
 class ShinnyLogger(logging.Logger):
 
     def debug(self, msg, *args, **kwargs):
@@ -70,7 +72,9 @@ class ShinnyLogger(logging.Logger):
 
 logging.setLoggerClass(klass=ShinnyLogger)
 
-
+# ---------------------------------------------------------------------------
+#   JSONFormatter
+# ---------------------------------------------------------------------------
 _VALID_ATTRS_to_BUILTIN_ATTRS = {
     'created': "created",
     'file_name': 'filename',
@@ -136,6 +140,30 @@ def _json_serializable(obj):
         return str(obj)
 
 
+# ---------------------------------------------------------------------------
+#   ShinnyLoggerAdapter
+# ---------------------------------------------------------------------------
+class ShinnyLoggerAdapter(logging.LoggerAdapter):
+
+    def __init__(self, logger, **kwargs):
+        self.logger = logger
+        self.extra = kwargs
+
+    def process(self, msg, kwargs):
+        if "extra" in kwargs:
+            extra = self.extra.copy()
+            extra.update(kwargs["extra"])
+            kwargs["extra"] = extra
+        else:
+            kwargs["extra"] = self.extra
+        return msg, kwargs
+
+    def bind(self, **kwargs):
+        extra = self.extra.copy()
+        extra.update(kwargs)
+        return ShinnyLoggerAdapter(self.logger, **extra)
+
+
 root = ShinnyLogger(logging.WARNING)
 root.name = 'root'
 logging.Logger.root = root
